@@ -1,8 +1,26 @@
-/* $Xorg: Converters.c,v 1.5 2001/02/09 02:03:54 xorgcvs Exp $ */
-
 /***********************************************************
-Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts
-Copyright 1993 by Sun Microsystems, Inc. Mountain View, CA.
+Copyright (c) 1993, Oracle and/or its affiliates. All rights reserved.
+
+Permission is hereby granted, free of charge, to any person obtaining a
+copy of this software and associated documentation files (the "Software"),
+to deal in the Software without restriction, including without limitation
+the rights to use, copy, modify, merge, publish, distribute, sublicense,
+and/or sell copies of the Software, and to permit persons to whom the
+Software is furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice (including the next
+paragraph) shall be included in all copies or substantial portions of the
+Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+DEALINGS IN THE SOFTWARE.
+
+Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts.
 
                         All Rights Reserved
 
@@ -10,7 +28,7 @@ Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
 both that copyright notice and this permission notice appear in
-supporting documentation, and that the names of Digital or Sun not be
+supporting documentation, and that the name of Digital not be
 used in advertising or publicity pertaining to distribution of the
 software without specific, written prior permission.
 
@@ -22,17 +40,7 @@ WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
-SUN DISCLAIMS ALL WARRANTIES WITH REGARD TO  THIS  SOFTWARE,
-INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
-NESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL SUN BE  LI-
-ABLE  FOR  ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
-ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,  DATA  OR
-PROFITS,  WHETHER  IN  AN  ACTION OF CONTRACT, NEGLIGENCE OR
-OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
-THE USE OR PERFORMANCE OF THIS SOFTWARE.
-
 ******************************************************************/
-/* $XFree86: xc/lib/Xt/Converters.c,v 3.14tsi Exp $ */
 
 /*
 
@@ -75,17 +83,12 @@ in this Software without prior written authorization from The Open Group.
 #include	<X11/Xlocale.h>
 #include	<errno.h>	/* for StringToDirectoryString */
 
-#ifdef __UNIXOS2__
-#define IsNewline(str) ((str) == '\n' || (str) == '\r')
-#define IsWhitespace(str) ((str)== ' ' || (str) == '\t' || (str) == '\r')
-#else
 #define IsNewline(str) ((str) == '\n')
 #define IsWhitespace(str) ((str)== ' ' || (str) == '\t')
-#endif
 
-static const String XtNwrongParameters = "wrongParameters";
-static const String XtNconversionError = "conversionError";
-static const String XtNmissingCharsetList = "missingCharsetList";
+static _Xconst _XtString XtNwrongParameters = "wrongParameters";
+static _Xconst _XtString XtNconversionError = "conversionError";
+static _Xconst _XtString XtNmissingCharsetList = "missingCharsetList";
 
 /* Representation types */
 
@@ -114,11 +117,7 @@ static XrmQuark  XtQGravity;
 static XrmQuark  XtQInt;
 static XrmQuark  XtQPixel;
 static XrmQuark  XtQPosition;
-#ifdef __UNIXOS2__
-XrmQuark  _XtQString = 0;
-#else
 XrmQuark  _XtQString;
-#endif
 
 void _XtConvertInitialize(void)
 {
@@ -145,11 +144,11 @@ void _XtConvertInitialize(void)
 			(char*) fromVal->addr, tstr);		\
 		    return False;				\
 		}						\
-		*(type*)(toVal->addr) = (value);		\
+		*(type*)(toVal->addr) = (type) (value);		\
 	    }							\
 	    else {						\
 		static type static_val;				\
-		static_val = (value);				\
+		static_val = (type) (value);			\
 		toVal->addr = (XPointer)&static_val;		\
 	    }							\
 	    toVal->size = sizeof(type);				\
@@ -163,11 +162,11 @@ void _XtConvertInitialize(void)
 		    toVal->size = sizeof(type);			\
 		    return False;				\
 		}						\
-		*(type*)(toVal->addr) = (value);		\
+		*(type*)(toVal->addr) = (type) (value);		\
 	    }							\
 	    else {						\
 		static type static_val;				\
-		static_val = (value);				\
+		static_val = (type) (value);			\
 		toVal->addr = (XPointer)&static_val;		\
 	    }							\
 	    toVal->size = sizeof(type);				\
@@ -249,7 +248,7 @@ void XtStringConversionWarning(
 		    params,&num_params);
 }
 
-static int CompareISOLatin1(char *, char *);
+static int CompareISOLatin1(const char *, const char *);
 
 
 static Boolean IsInteger(
@@ -262,11 +261,7 @@ static Boolean IsInteger(
     int val = 0;
     char ch;
     /* skip leading whitespace */
-#ifndef __UNIXOS2__
     while ((ch = *string) == ' ' || ch == '\t') string++;
-#else
-    while ((ch = *string) == ' ' || ch == '\t' || ch == '\r') string++;
-#endif
     while ((ch = *string++)) {
 	if (ch >= '0' && ch <= '9') {
 	    val *= 10;
@@ -317,7 +312,7 @@ Boolean XtCvtIntToBoolean(
 	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 		  XtNwrongParameters,"cvtIntToBoolean",XtCXtToolkitError,
                   "Integer to Boolean conversion needs no extra arguments",
-                   (String *)NULL, (Cardinal *)NULL);
+                   NULL, NULL);
     done(Boolean, (*(int *)fromVal->addr != 0));
 }
 
@@ -335,7 +330,7 @@ Boolean XtCvtIntToShort(
 	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 		  XtNwrongParameters,"cvtIntToShort",XtCXtToolkitError,
                   "Integer to Short conversion needs no extra arguments",
-                   (String *)NULL, (Cardinal *)NULL);
+                   NULL, NULL);
     done(short, (*(int *)fromVal->addr));
 }
 
@@ -354,7 +349,7 @@ Boolean XtCvtStringToBoolean(
 	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 		  XtNwrongParameters,"cvtStringToBoolean",XtCXtToolkitError,
                   "String to Boolean conversion needs no extra arguments",
-                   (String *)NULL, (Cardinal *)NULL);
+                   NULL, NULL);
 
     if (   (CompareISOLatin1(str, "true") == 0)
 	|| (CompareISOLatin1(str, "yes") == 0)
@@ -384,7 +379,7 @@ Boolean XtCvtIntToBool(
 	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 		  XtNwrongParameters,"cvtIntToBool",XtCXtToolkitError,
                   "Integer to Bool conversion needs no extra arguments",
-                   (String *)NULL, (Cardinal *)NULL);
+                   NULL, NULL);
     done(Bool, (*(int *)fromVal->addr != 0));
 }
 
@@ -404,7 +399,7 @@ Boolean XtCvtStringToBool(
 		XtNwrongParameters,"cvtStringToBool",
 		XtCXtToolkitError,
                  "String to Bool conversion needs no extra arguments",
-                  (String *)NULL, (Cardinal *)NULL);
+                  NULL, NULL);
 
     if (   (CompareISOLatin1(str, "true") == 0)
 	|| (CompareISOLatin1(str, "yes") == 0)
@@ -445,12 +440,12 @@ Boolean XtCvtIntToColor(
       XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 	 XtNwrongParameters,"cvtIntOrPixelToXColor",XtCXtToolkitError,
          "Pixel to color conversion needs screen and colormap arguments",
-          (String *)NULL, (Cardinal *)NULL);
+          NULL, NULL);
       return False;
     }
     screen = *((Screen **) args[0].addr);
     colormap = *((Colormap *) args[1].addr);
-    c.pixel = *(int *)fromVal->addr;
+    c.pixel = (unsigned long) (*(int *)fromVal->addr);
 
     XQueryColor(DisplayOfScreen(screen), colormap, &c);
     done(XColor, c);
@@ -472,14 +467,13 @@ Boolean XtCvtStringToPixel(
     XtPerDisplay    pd = _XtGetPerDisplay(dpy);
     Colormap	    colormap;
     Status	    status;
-    String          params[1];
     Cardinal	    num_params=1;
 
     if (*num_args != 2) {
      XtAppWarningMsg(pd->appContext, XtNwrongParameters, "cvtStringToPixel",
 		     XtCXtToolkitError,
 	"String to pixel conversion needs screen and colormap arguments",
-        (String *)NULL, (Cardinal *)NULL);
+        NULL, NULL);
      return False;
     }
 
@@ -487,12 +481,12 @@ Boolean XtCvtStringToPixel(
     colormap = *((Colormap *) args[1].addr);
 
     if (CompareISOLatin1(str, XtDefaultBackground) == 0) {
-	*closure_ret = False;
+	*closure_ret = NULL;
 	if (pd->rv) donestr(Pixel, BlackPixelOfScreen(screen), XtRPixel)
 	else	    donestr(Pixel, WhitePixelOfScreen(screen), XtRPixel);
     }
     if (CompareISOLatin1(str, XtDefaultForeground) == 0) {
-	*closure_ret = False;
+	*closure_ret = NULL;
 	if (pd->rv) donestr(Pixel, WhitePixelOfScreen(screen), XtRPixel)
         else	    donestr(Pixel, BlackPixelOfScreen(screen), XtRPixel);
     }
@@ -500,7 +494,10 @@ Boolean XtCvtStringToPixel(
     status = XAllocNamedColor(DisplayOfScreen(screen), colormap,
 			      (char*)str, &screenColor, &exactColor);
     if (status == 0) {
-	String msg, type;
+	_Xconst _XtString msg;
+	_Xconst _XtString type;
+	String params[1];
+
 	params[0] = str;
 	/* Server returns a specific error code but Xlib discards it.  Ugh */
 	if (XLookupColor(DisplayOfScreen(screen), colormap, (char*)str,
@@ -515,7 +512,7 @@ Boolean XtCvtStringToPixel(
 
 	XtAppWarningMsg(pd->appContext, type, "cvtStringToPixel",
 			XtCXtToolkitError, msg, params, &num_params);
-	*closure_ret = False;
+	*closure_ret = NULL;
 	return False;
     } else {
 	*closure_ret = (char*)True;
@@ -537,7 +534,7 @@ static void FreePixel(
     if (*num_args != 2) {
      XtAppWarningMsg(app, XtNwrongParameters,"freePixel",XtCXtToolkitError,
 	"Freeing a pixel requires screen and colormap arguments",
-        (String *)NULL, (Cardinal *)NULL);
+        NULL, NULL);
      return;
     }
 
@@ -564,15 +561,16 @@ static void FetchDisplayArg(
     Cardinal *size,
     XrmValue* value)
 {
-    if (widget == NULL)
+    if (widget == NULL) {
 	XtErrorMsg("missingWidget", "fetchDisplayArg", XtCXtToolkitError,
 		   "FetchDisplayArg called without a widget to reference",
-		   (String*)NULL, (Cardinal*)NULL);
+		   NULL, NULL);
         /* can't return any useful Display and caller will de-ref NULL,
 	   so aborting is the only useful option */
-
-    value->size = sizeof(Display*);
-    value->addr = (XPointer)&DisplayOfScreen(XtScreenOfObject(widget));
+    } else {
+	value->size = sizeof(Display*);
+	value->addr = (XPointer)&DisplayOfScreen(XtScreenOfObject(widget));
+    }
 }
 
 static XtConvertArgRec const displayConvertArg[] = {
@@ -678,7 +676,7 @@ Boolean XtCvtStringToCursor(
 	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 	     XtNwrongParameters,"cvtStringToCursor",XtCXtToolkitError,
              "String to cursor conversion needs display argument",
-              (String *)NULL, (Cardinal *)NULL);
+              NULL, NULL);
 	return False;
     }
 
@@ -707,7 +705,7 @@ static void FreeCursor(
      XtAppWarningMsg(app,
 	     XtNwrongParameters,"freeCursor",XtCXtToolkitError,
              "Free Cursor requires display argument",
-              (String *)NULL, (Cardinal *)NULL);
+              NULL, NULL);
      return;
     }
 
@@ -730,7 +728,7 @@ Boolean XtCvtStringToDisplay(
 	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 		  XtNwrongParameters,"cvtStringToDisplay",XtCXtToolkitError,
                   "String to Display conversion needs no extra arguments",
-                   (String *)NULL, (Cardinal *)NULL);
+                   NULL, NULL);
 
     d = XOpenDisplay((char *)fromVal->addr);
     if (d != NULL)
@@ -756,7 +754,7 @@ Boolean XtCvtStringToFile(
 	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 		 XtNwrongParameters,"cvtStringToFile",XtCXtToolkitError,
                  "String to File conversion needs no extra arguments",
-                 (String *) NULL, (Cardinal *)NULL);
+                 NULL, NULL);
 
     f = fopen((char *)fromVal->addr, "r");
     if (f != NULL)
@@ -778,7 +776,7 @@ static void FreeFile(
 	XtAppWarningMsg(app,
 		 XtNwrongParameters,"freeFile",XtCXtToolkitError,
                  "Free File requires no extra arguments",
-                 (String *) NULL, (Cardinal *)NULL);
+                 NULL, NULL);
 
     fclose( *(FILE**)toVal->addr );
 }
@@ -796,7 +794,7 @@ Boolean XtCvtIntToFloat(
 	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 		  XtNwrongParameters,"cvtIntToFloat",XtCXtToolkitError,
                   "Integer to Float conversion needs no extra arguments",
-                   (String *)NULL, (Cardinal *)NULL);
+                   NULL, NULL);
     done(float, (*(int *)fromVal->addr));
 }
 
@@ -812,17 +810,14 @@ Boolean XtCvtStringToFloat(
     int ret;
     float f, nan;
 
-#ifndef ISC /* On ISC this generates a core dump :-( at least with gs */
-    /* depending on the system this may or may not do anything useful */
-    (void) sscanf ("NaNS", "%g",
+    (void) sscanf ("NaN", "%g",
 		   toVal->addr != NULL ? (float*) toVal->addr : &nan);
-#endif
 
     if (*num_args != 0)
 	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 		 XtNwrongParameters,"cvtStringToFloat",XtCXtToolkitError,
                  "String to Float conversion needs no extra arguments",
-                 (String *) NULL, (Cardinal *)NULL);
+                 NULL, NULL);
 
     ret = sscanf (fromVal->addr, "%g", &f);
     if (ret == 0) {
@@ -850,7 +845,7 @@ Boolean XtCvtStringToFont(
 	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 	     XtNwrongParameters,"cvtStringToFont",XtCXtToolkitError,
              "String to font conversion needs display argument",
-              (String *) NULL, (Cardinal *)NULL);
+              NULL, NULL);
 	return False;
     }
 
@@ -901,7 +896,7 @@ Boolean XtCvtStringToFont(
     XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 		    "noFont","cvtStringToFont",XtCXtToolkitError,
 		    "Unable to load any usable ISO8859 font",
-		    (String *) NULL, (Cardinal *)NULL);
+		    NULL, NULL);
 
     return False;
 }
@@ -919,7 +914,7 @@ static void FreeFont(
 	XtAppWarningMsg(app,
 	     XtNwrongParameters,"freeFont",XtCXtToolkitError,
              "Free Font needs display argument",
-              (String *) NULL, (Cardinal *)NULL);
+              NULL, NULL);
 	return;
     }
 
@@ -940,7 +935,7 @@ Boolean XtCvtIntToFont(
 	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 	   XtNwrongParameters,"cvtIntToFont",XtCXtToolkitError,
            "Integer to Font conversion needs no extra arguments",
-            (String *) NULL, (Cardinal *)NULL);
+            NULL, NULL);
     done(Font, *(int*)fromVal->addr);
 }
 
@@ -963,7 +958,7 @@ Boolean XtCvtStringToFontSet(
       XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
            XtNwrongParameters,"cvtStringToFontSet",XtCXtToolkitError,
              "String to FontSet conversion needs display and locale arguments",
-              (String *) NULL, (Cardinal *)NULL);
+              NULL, NULL);
       return False;
     }
 
@@ -977,7 +972,7 @@ Boolean XtCvtStringToFontSet(
           XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
                  XtNmissingCharsetList,"cvtStringToFontSet",XtCXtToolkitError,
                  "Missing charsets in String to FontSet conversion",
-                 (String *) NULL, (Cardinal *)NULL);
+                 NULL, NULL);
             XFreeStringList(missing_charset_list);
       }
       if (f != NULL) {
@@ -1010,7 +1005,7 @@ Boolean XtCvtStringToFontSet(
 			 XtNmissingCharsetList,"cvtStringToFontSet",
 			 XtCXtToolkitError,
 			 "Missing charsets in String to FontSet conversion",
-                         (String *) NULL, (Cardinal *)NULL);
+                         NULL, NULL);
 		  XFreeStringList(missing_charset_list);
               }
               if (f != NULL)
@@ -1034,7 +1029,7 @@ Boolean XtCvtStringToFontSet(
       XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
              XtNmissingCharsetList,"cvtStringToFontSet",XtCXtToolkitError,
              "Missing charsets in String to FontSet conversion",
-             (String *) NULL, (Cardinal *)NULL);
+             NULL, NULL);
         XFreeStringList(missing_charset_list);
     }
     if (f != NULL)
@@ -1043,7 +1038,7 @@ Boolean XtCvtStringToFontSet(
     XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
            "noFont","cvtStringToFontSet",XtCXtToolkitError,
              "Unable to load any usable fontset",
-              (String *) NULL, (Cardinal *)NULL);
+              NULL, NULL);
 
     return False;
 }
@@ -1061,7 +1056,7 @@ static void FreeFontSet(
       XtAppWarningMsg(app,
            XtNwrongParameters,"freeFontSet",XtCXtToolkitError,
              "FreeFontSet needs display and locale arguments",
-              (String *) NULL, (Cardinal *)NULL);
+              NULL, NULL);
       return;
     }
 
@@ -1106,7 +1101,7 @@ XtCvtStringToFontStruct(
      XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 	     XtNwrongParameters,"cvtStringToFontStruct",XtCXtToolkitError,
              "String to font conversion needs display argument",
-              (String *) NULL, (Cardinal *)NULL);
+              NULL, NULL);
      return False;
     }
 
@@ -1160,7 +1155,7 @@ XtCvtStringToFontStruct(
     XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 	     "noFont","cvtStringToFontStruct",XtCXtToolkitError,
              "Unable to load any usable ISO8859 font",
-              (String *) NULL, (Cardinal *)NULL);
+              NULL, NULL);
 
     return False;
 }
@@ -1178,7 +1173,7 @@ static void FreeFontStruct(
      XtAppWarningMsg(app,
 	     XtNwrongParameters,"freeFontStruct",XtCXtToolkitError,
              "Free FontStruct requires display argument",
-              (String *) NULL, (Cardinal *)NULL);
+              NULL, NULL);
      return;
     }
 
@@ -1201,7 +1196,7 @@ Boolean XtCvtStringToInt(
 	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 		  XtNwrongParameters,"cvtStringToInt",XtCXtToolkitError,
                   "String to Integer conversion needs no extra arguments",
-                  (String *) NULL, (Cardinal *)NULL);
+                  NULL, NULL);
     if (IsInteger((String)fromVal->addr, &i))
 	donestr(int, i, XtRInt);
 
@@ -1224,7 +1219,7 @@ Boolean XtCvtStringToShort(
         XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 	  XtNwrongParameters,"cvtStringToShort",XtCXtToolkitError,
           "String to Integer conversion needs no extra arguments",
-           (String *) NULL, (Cardinal *)NULL);
+           NULL, NULL);
     if (IsInteger((String)fromVal->addr, &i))
         donestr(short, (short)i, XtRShort);
 
@@ -1247,7 +1242,7 @@ Boolean XtCvtStringToDimension(
         XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 	  XtNwrongParameters,"cvtStringToDimension",XtCXtToolkitError,
           "String to Dimension conversion needs no extra arguments",
-           (String *) NULL, (Cardinal *)NULL);
+           NULL, NULL);
     if (IsInteger((String)fromVal->addr, &i)) {
         if ( i < 0 )
             XtDisplayStringConversionWarning(dpy, (char*)fromVal->addr,
@@ -1271,7 +1266,7 @@ Boolean XtCvtIntToUnsignedChar(
 	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 		  XtNwrongParameters,"cvtIntToUnsignedChar",XtCXtToolkitError,
                   "Integer to UnsignedChar conversion needs no extra arguments",
-                   (String *)NULL, (Cardinal *)NULL);
+                   NULL, NULL);
     done(unsigned char, (*(int *)fromVal->addr));
 }
 
@@ -1291,7 +1286,7 @@ Boolean XtCvtStringToUnsignedChar(
         XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 		  XtNwrongParameters,"cvtStringToUnsignedChar",XtCXtToolkitError,
                   "String to Integer conversion needs no extra arguments",
-                   (String *) NULL, (Cardinal *)NULL);
+                   NULL, NULL);
     if (IsInteger((String)fromVal->addr, &i)) {
         if ( i < 0 || i > 255 )
             XtDisplayStringConversionWarning(dpy, (char*)fromVal->addr,
@@ -1317,7 +1312,7 @@ Boolean XtCvtColorToPixel(
 	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 		  XtNwrongParameters,"cvtXColorToPixel",XtCXtToolkitError,
                   "Color to Pixel conversion needs no extra arguments",
-                   (String *) NULL, (Cardinal *)NULL);
+                   NULL, NULL);
     done(Pixel, ((XColor *)fromVal->addr)->pixel);
 }
 
@@ -1334,7 +1329,7 @@ Boolean XtCvtIntToPixel(
 	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 		  XtNwrongParameters,"cvtIntToPixel",XtCXtToolkitError,
                   "Integer to Pixel conversion needs no extra arguments",
-                   (String *) NULL, (Cardinal *)NULL);
+                   NULL, NULL);
     done(Pixel, *(int*)fromVal->addr);
 }
 
@@ -1351,8 +1346,8 @@ Boolean XtCvtIntToPixmap(
         XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 		  XtNwrongParameters,"cvtIntToPixmap",XtCXtToolkitError,
                   "Integer to Pixmap conversion needs no extra arguments",
-                   (String *) NULL, (Cardinal *)NULL);
-    done(Pixmap, *(Pixmap*)fromVal->addr);
+                   NULL, NULL);
+    done(Pixmap, *(int*)fromVal->addr);
 }
 
 #ifdef MOTIFBC
@@ -1371,11 +1366,12 @@ void LowerCase(register char  *source, register *dest)
 }
 #endif
 
-static int CompareISOLatin1 (char *first, char *second)
+static int CompareISOLatin1 (const char *first, const char *second)
 {
-    register unsigned char *ap, *bp;
+    register const unsigned char *ap, *bp;
 
-    for (ap = (unsigned char *) first, bp = (unsigned char *) second;
+    for (ap = (const unsigned char *) first,
+	 bp = (const unsigned char *) second;
 	 *ap && *bp; ap++, bp++) {
 	register unsigned char a, b;
 
@@ -1383,18 +1379,18 @@ static int CompareISOLatin1 (char *first, char *second)
 	    /* try lowercasing and try again */
 
 	    if ((a >= XK_A) && (a <= XK_Z))
-	      a += (XK_a - XK_A);
+	      a = (unsigned char) (a + (XK_a - XK_A));
 	    else if ((a >= XK_Agrave) && (a <= XK_Odiaeresis))
-	      a += (XK_agrave - XK_Agrave);
+	      a = (unsigned char) (a + (XK_agrave - XK_Agrave));
 	    else if ((a >= XK_Ooblique) && (a <= XK_Thorn))
-	      a += (XK_oslash - XK_Ooblique);
+	      a = (unsigned char) (a + (XK_oslash - XK_Ooblique));
 
 	    if ((b >= XK_A) && (b <= XK_Z))
-	      b += (XK_a - XK_A);
+	      b = (unsigned char) (b + (XK_a - XK_A));
 	    else if ((b >= XK_Agrave) && (b <= XK_Odiaeresis))
-	      b += (XK_agrave - XK_Agrave);
+	      b = (unsigned char) (b + (XK_agrave - XK_Agrave));
 	    else if ((b >= XK_Ooblique) && (b <= XK_Thorn))
-	      b += (XK_oslash - XK_Ooblique);
+	      b = (unsigned char) (b + (XK_oslash - XK_Ooblique));
 
 	    if (a != b) break;
 	}
@@ -1402,19 +1398,18 @@ static int CompareISOLatin1 (char *first, char *second)
     return (((int) *bp) - ((int) *ap));
 }
 
-static void CopyISOLatin1Lowered(char *dst, char *src)
+static void CopyISOLatin1Lowered(char *dst, const char *src)
 {
-    unsigned char *dest, *source;
-
-    dest = (unsigned char *) dst; source = (unsigned char *) src;
+    unsigned char *dest = (unsigned char *) dst;
+    const unsigned char *source = (const unsigned char *) src;
 
     for ( ; *source; source++, dest++) {
 	if (*source >= XK_A  && *source <= XK_Z)
-	    *dest = *source + (XK_a - XK_A);
+	    *dest = (unsigned char) (*source + (XK_a - XK_A));
 	else if (*source >= XK_Agrave && *source <= XK_Odiaeresis)
-	    *dest = *source + (XK_agrave - XK_Agrave);
+	    *dest = (unsigned char) (*source + (XK_agrave - XK_Agrave));
 	else if (*source >= XK_Ooblique && *source <= XK_Thorn)
-	    *dest = *source + (XK_oslash - XK_Ooblique);
+	    *dest = (unsigned char) (*source + (XK_oslash - XK_Ooblique));
 	else
 	    *dest = *source;
     }
@@ -1436,7 +1431,7 @@ XtCvtStringToInitialState(
 	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 		  XtNwrongParameters,"cvtStringToInitialState",XtCXtToolkitError,
                   "String to InitialState conversion needs no extra arguments",
-                   (String *) NULL, (Cardinal *)NULL);
+                   NULL, NULL);
 
     if (CompareISOLatin1(str, "NormalState") == 0) donestr(int, NormalState, XtRInitialState);
     if (CompareISOLatin1(str, "IconicState") == 0) donestr(int, IconicState, XtRInitialState);
@@ -1471,7 +1466,7 @@ Boolean XtCvtStringToVisual(
 	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 		  XtNwrongParameters,"cvtStringToVisual",XtCXtToolkitError,
                   "String to Visual conversion needs screen and depth arguments",
-                   (String *) NULL, (Cardinal *)NULL);
+                   NULL, NULL);
 	return False;
     }
 
@@ -1522,7 +1517,7 @@ Boolean XtCvtStringToAtom(
 	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 		  XtNwrongParameters,"cvtStringToAtom",XtCXtToolkitError,
                   "String to Atom conversion needs Display argument",
-                   (String *) NULL, (Cardinal *)NULL);
+                   NULL, NULL);
 	return False;
     }
 
@@ -1546,7 +1541,7 @@ Boolean XtCvtStringToDirectoryString(
 	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
            XtNwrongParameters,"cvtStringToDirectoryString",XtCXtToolkitError,
            "String to DirectoryString conversion needs no extra arguments",
-           (String *)NULL, (Cardinal *)NULL);
+           NULL, NULL);
 
     str = (String)fromVal->addr;
     if (CompareISOLatin1(str, "XtCurrentDirectory") == 0) {
@@ -1587,7 +1582,7 @@ static void FreeDirectoryString(
 	XtAppWarningMsg(app,
 		 XtNwrongParameters,"freeDirectoryString",XtCXtToolkitError,
 		 "Free Directory String requires no extra arguments",
-                 (String *) NULL, (Cardinal *) NULL);
+                 NULL, NULL);
 
     XtFree((char *) toVal->addr);
 }
@@ -1606,7 +1601,7 @@ Boolean XtCvtStringToRestartStyle(
 	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
 	      XtNwrongParameters,"cvtStringToRestartStyle",XtCXtToolkitError,
               "String to RestartStyle conversion needs no extra arguments",
-              (String *)NULL, (Cardinal *)NULL);
+              NULL, NULL);
 
     if (CompareISOLatin1(str, "RestartIfRunning") == 0)
 	donestr(unsigned char, SmRestartIfRunning, XtRRestartStyle);
@@ -1632,20 +1627,21 @@ Boolean XtCvtStringToCommandArgArray(
     String *strarray, *ptr;
     char *src;
     char *dst, *dst_str;
-    char *start;
     int tokens, len;
 
     if (*num_args != 0)
 	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
             XtNwrongParameters,"cvtStringToCommandArgArray",XtCXtToolkitError,
             "String to CommandArgArray conversion needs no extra arguments",
-            (String *)NULL, (Cardinal *)NULL);
+            NULL, NULL);
 
     src = fromVal->addr;
     dst = dst_str = __XtMalloc((unsigned) strlen(src) + 1);
     tokens = 0;
 
     while (*src != '\0') {
+	char *start;
+
 	/* skip whitespace */
 	while (IsWhitespace(*src) || IsNewline(*src))
 	    src++;
@@ -1659,10 +1655,10 @@ Boolean XtCvtStringToCommandArgArray(
 	while (*src != '\0' && !IsWhitespace(*src) && !IsNewline(*src)) {
 	    if (*src == '\\' &&
 		(IsWhitespace(*(src+1)) || IsNewline(*(src+1)))) {
-		len = src - start;
+		len = (int) (src - start);
 		if (len) {
 		    /* copy preceeding part of token */
-		    memcpy(dst, start, len);
+		    memcpy(dst, start, (size_t) len);
 		    dst += len;
 		}
 		/* skip backslash */
@@ -1672,10 +1668,10 @@ Boolean XtCvtStringToCommandArgArray(
 	    }
 	    src++;
 	}
-	len = src - start;
+	len = (int) (src - start);
 	if (len) {
 	    /* copy last part of token */
-	    memcpy(dst, start, len);
+	    memcpy(dst, start, (size_t) len);
 	    dst += len;
 	}
 	*dst = '\0';
@@ -1683,13 +1679,13 @@ Boolean XtCvtStringToCommandArgArray(
 	    dst++;
     }
 
-    ptr = strarray = (String*) __XtMalloc((Cardinal)(tokens+1) * sizeof(String));
+    ptr = strarray = (String*) __XtMalloc((Cardinal)((size_t)(tokens+1) * sizeof(String)));
     src = dst_str;
     while (--tokens >= 0) {
 	*ptr = src;
 	ptr++;
 	if (tokens) {
-	    len = strlen(src);
+	    len = (int) strlen(src);
 	    src = src + len + 1;
 	}
     }
@@ -1707,10 +1703,8 @@ static void ArgArrayDestructor(
     XrmValuePtr	args,
     Cardinal	*num_args)
 {
-    String *strarray;
-
     if (closure) {
-	strarray = (String*) closure;
+	_XtString *strarray = (_XtString*) closure;
 	XtFree(*strarray);
 	XtFree((char *) strarray);
     }
@@ -1727,7 +1721,7 @@ Boolean XtCvtStringToGravity (
 {
     static struct _namepair {
 	XrmQuark quark;
-	char *name;
+	const char *name;
 	int gravity;
     } names[] = {
 	{ NULLQUARK, "forget",		ForgetGravity },
@@ -1765,7 +1759,7 @@ Boolean XtCvtStringToGravity (
         XtAppWarningMsg(XtDisplayToApplicationContext (dpy),
 			"wrongParameters","cvtStringToGravity","XtToolkitError",
 			"String to Gravity conversion needs no extra arguments",
-			(String *) NULL, (Cardinal *)NULL);
+			NULL, NULL);
 	return False;
     }
     if (!haveQuarks) {
