@@ -1,8 +1,26 @@
-/* $Xorg: Error.c,v 1.5 2001/02/09 02:03:54 xorgcvs Exp $ */
-
 /***********************************************************
-Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts
-Copyright 1993 by Sun Microsystems, Inc. Mountain View, CA.
+Copyright (c) 1993, Oracle and/or its affiliates. All rights reserved.
+
+Permission is hereby granted, free of charge, to any person obtaining a
+copy of this software and associated documentation files (the "Software"),
+to deal in the Software without restriction, including without limitation
+the rights to use, copy, modify, merge, publish, distribute, sublicense,
+and/or sell copies of the Software, and to permit persons to whom the
+Software is furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice (including the next
+paragraph) shall be included in all copies or substantial portions of the
+Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+DEALINGS IN THE SOFTWARE.
+
+Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts.
 
                         All Rights Reserved
 
@@ -10,7 +28,7 @@ Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
 both that copyright notice and this permission notice appear in
-supporting documentation, and that the names of Digital or Sun not be
+supporting documentation, and that the name of Digital not be
 used in advertising or publicity pertaining to distribution of the
 software without specific, written prior permission.
 
@@ -22,17 +40,7 @@ WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
-SUN DISCLAIMS ALL WARRANTIES WITH REGARD TO  THIS  SOFTWARE,
-INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
-NESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL SUN BE  LI-
-ABLE  FOR  ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
-ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,  DATA  OR
-PROFITS,  WHETHER  IN  AN  ACTION OF CONTRACT, NEGLIGENCE OR
-OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
-THE USE OR PERFORMANCE OF THIS SOFTWARE.
-
 ******************************************************************/
-/* $XFree86: xc/lib/Xt/Error.c,v 3.13tsi Exp $ */
 
 /*
 
@@ -81,11 +89,11 @@ static XrmDatabase errorDB = NULL;
 static Boolean error_inited = FALSE;
 void _XtDefaultErrorMsg(String, String, String, String, String*, Cardinal*);
 void _XtDefaultWarningMsg(String, String, String, String, String*, Cardinal*);
-void _XtDefaultError(String);
+void _XtDefaultError(String) _X_NORETURN;
 void _XtDefaultWarning(String);
 static XtErrorMsgHandler errorMsgHandler = _XtDefaultErrorMsg;
 static XtErrorMsgHandler warningMsgHandler = _XtDefaultWarningMsg;
-static XtErrorHandler errorHandler = _XtDefaultError;
+static XtErrorHandler errorHandler _X_NORETURN = _XtDefaultError;
 static XtErrorHandler warningHandler = _XtDefaultWarning;
 #endif /* GLOBALERRORS */
 
@@ -123,7 +131,7 @@ void XtGetErrorDatabaseText(
     register _Xconst char* type,
     register _Xconst char* class,
     _Xconst char* defaultp,
-    String buffer,
+    _XtString buffer,
     int nbytes)
 {
 #if GLOBALERRORS
@@ -141,12 +149,12 @@ void XtAppGetErrorDatabaseText(
     register _Xconst char* type,
     register _Xconst char* class,
     _Xconst char* defaultp,
-    String buffer,
+    _XtString buffer,
     int nbytes,
     XrmDatabase db)
 {
     String str_class;
-    String type_str;
+    _XtString type_str;
     XrmValue result;
     char *str_name = NULL;
     char *temp = NULL;
@@ -186,12 +194,12 @@ void XtAppGetErrorDatabaseText(
 #endif /* GLOBALERRORS */
     } else (void) XrmGetResource(db, str_name, str_class, &type_str, &result);
     if (result.addr) {
-        (void) strncpy (buffer, result.addr, nbytes);
+        (void) strncpy (buffer, result.addr, (size_t) nbytes);
         if (result.size > (unsigned) nbytes) buffer[nbytes-1] = 0;
     } else {
-	int len = strlen(defaultp);
+	int len = (int) strlen(defaultp);
 	if (len >= nbytes) len = nbytes-1;
-	(void) memmove(buffer, defaultp, len);
+	(void) memmove(buffer, defaultp, (size_t) len);
 	buffer[len] = '\0';
     }
     if (str_name)
@@ -225,11 +233,6 @@ static void DefaultMsg (
     void (*fn)(_Xconst _XtString))
 {
 #define BIGBUF 1024
-#ifdef notyet /* older versions don't, might want to wait until more do */
-#if defined(__linux__)  || defined(CSRG_BASED) /* everyone else needs to get with the program */
-#define USE_SNPRINTF
-#endif
-#endif
     char buffer[BIGBUF];
     char* message;
     XtGetErrorDatabaseText(name,type,class,defaultp, buffer, BIGBUF);
@@ -248,7 +251,7 @@ static void DefaultMsg (
 	    String par[10];
 	    if (i > 10) i = 10;
 	    (void) memmove((char*)par, (char*)params, i * sizeof(String) );
-	    bzero( &par[i], (10-i) * sizeof(String) );
+	    memset( &par[i], 0, (10-i) * sizeof(String) );
 	    (void) fprintf (stderr, "%s%s",
 			    error ? XTERROR_PREFIX : XTWARNING_PREFIX,
 			    error ? "Error: " : "Warning: ");
@@ -283,7 +286,7 @@ program as a non-root user or by removing the suid bit on the executable.");
 	String par[10];
 	if (i > 10) i = 10;
 	(void) memmove((char*)par, (char*)params, i * sizeof(String) );
-	bzero( &par[i], (10-i) * sizeof(String) );
+	memset( &par[i], 0, (10-i) * sizeof(String) );
 	if (i != *num_params)
 	    XtWarning( "Some arguments in following message were lost" );
 	/*
@@ -294,18 +297,9 @@ program as a non-root user or by removing the suid bit on the executable.");
 	 * to be a performance issue.
 	 */
 	if ((message = __XtMalloc (BIGBUF))) {
-#ifndef USE_SNPRINTF
-	    message[BIGBUF-1] = 0;
-	    (void) sprintf (message, buffer,
-#else
 	    (void) snprintf (message, BIGBUF, buffer,
-#endif
-			    par[0], par[1], par[2], par[3], par[4],
-			    par[5], par[6], par[7], par[8], par[9]);
-#ifndef USE_SNPRINTF
-	    if (message[BIGBUF-1] != '\0')
-		XtWarning ("Possible heap corruption in Xt{Error,Warning}MsgHandler");
-#endif
+			     par[0], par[1], par[2], par[3], par[4],
+			     par[5], par[6], par[7], par[8], par[9]);
 	    (*fn)(message);
 	    XtFree(message);
 	} else {
@@ -350,6 +344,7 @@ void XtErrorMsg(
     (*errorMsgHandler)((String)name,(String)type,(String)class,
 		       (String)defaultp,params,num_params);
     UNLOCK_PROCESS;
+    exit(1);
 #else
     XtAppErrorMsg(_XtDefaultAppContext(),name,type,class,
 	    defaultp,params,num_params);
@@ -370,6 +365,7 @@ void XtAppErrorMsg(
     (*errorMsgHandler)((String)name,(String)type,(String)class,
 		       (String)defaultp,params,num_params);
     UNLOCK_PROCESS;
+    exit(1);
 #else
     LOCK_APP(app);
     (*app->errorMsgHandler)(name,type,class,defaultp,params,num_params);
@@ -418,7 +414,7 @@ void XtAppWarningMsg(
 }
 
 void XtSetErrorMsgHandler(
-    XtErrorMsgHandler handler)
+    XtErrorMsgHandler handler _X_NORETURN)
 {
 #if GLOBALERRORS
     LOCK_PROCESS;
@@ -432,7 +428,7 @@ void XtSetErrorMsgHandler(
 
 XtErrorMsgHandler XtAppSetErrorMsgHandler(
     XtAppContext app,
-    XtErrorMsgHandler handler)
+    XtErrorMsgHandler handler _X_NORETURN)
 {
     XtErrorMsgHandler old;
 #if GLOBALERRORS
@@ -553,7 +549,7 @@ void XtAppWarning(
 #endif /* GLOBALERRORS */
 }
 
-void XtSetErrorHandler(XtErrorHandler handler)
+void XtSetErrorHandler(XtErrorHandler handler _X_NORETURN)
 {
 #if GLOBALERRORS
     LOCK_PROCESS;
@@ -567,7 +563,7 @@ void XtSetErrorHandler(XtErrorHandler handler)
 
 XtErrorHandler XtAppSetErrorHandler(
     XtAppContext app,
-    XtErrorHandler handler)
+    XtErrorHandler handler _X_NORETURN)
 {
     XtErrorHandler old;
 #if GLOBALERRORS
