@@ -1,8 +1,26 @@
-/* $Xorg: Event.c,v 1.5 2001/02/09 02:03:54 xorgcvs Exp $ */
-
 /***********************************************************
-Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts,
-Copyright 1993 by Sun Microsystems, Inc. Mountain View, CA.
+Copyright (c) 1993, Oracle and/or its affiliates. All rights reserved.
+
+Permission is hereby granted, free of charge, to any person obtaining a
+copy of this software and associated documentation files (the "Software"),
+to deal in the Software without restriction, including without limitation
+the rights to use, copy, modify, merge, publish, distribute, sublicense,
+and/or sell copies of the Software, and to permit persons to whom the
+Software is furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice (including the next
+paragraph) shall be included in all copies or substantial portions of the
+Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+DEALINGS IN THE SOFTWARE.
+
+Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts.
 
                         All Rights Reserved
 
@@ -10,7 +28,7 @@ Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
 both that copyright notice and this permission notice appear in
-supporting documentation, and that the names of Digital or Sun not be
+supporting documentation, and that the name of Digital not be
 used in advertising or publicity pertaining to distribution of the
 software without specific, written prior permission.
 
@@ -22,15 +40,6 @@ WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
-SUN DISCLAIMS ALL WARRANTIES WITH REGARD TO  THIS  SOFTWARE,
-INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
-NESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL SUN BE  LI-
-ABLE  FOR  ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
-ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,  DATA  OR
-PROFITS,  WHETHER  IN  AN  ACTION OF CONTRACT, NEGLIGENCE OR
-OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
-THE USE OR PERFORMANCE OF THIS SOFTWARE.
-
 ******************************************************************/
 
 /*
@@ -58,7 +67,6 @@ used in advertising or otherwise to promote the sale, use or other dealings
 in this Software without prior written authorization from The Open Group.
 
 */
-/* $XFree86: xc/lib/Xt/Event.c,v 3.10 2001/12/14 19:56:11 dawes Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -102,16 +110,17 @@ EventMask XtBuildEventMask(
     WIDGET_TO_APPCON(widget);
 
     LOCK_APP(app);
-    for (ev = widget->core.event_table; ev != NULL; ev = ev->next)
-	if (ev->select) {
-	    if (!ev->has_type_specifier)
-		mask |= ev->mask;
-	    else {
-		if (EXT_TYPE(ev) < LASTEvent) {
-		    Cardinal i;
-		    for (i = 0; i < ev->mask; i++)
-			if (EXT_SELECT_DATA(ev, i))
-			    mask |= *(EventMask*)EXT_SELECT_DATA(ev, i);
+    for (ev = widget->core.event_table; ev != NULL; ev = ev->next) {
+	if (! ev->select) continue;
+
+	if (!ev->has_type_specifier)
+	    mask |= ev->mask;
+	else {
+	    if (EXT_TYPE(ev) < LASTEvent) {
+		Cardinal i;
+		for (i = 0; i < ev->mask; i++)
+		    if (EXT_SELECT_DATA(ev, i))
+		        mask |= *(EventMask*)EXT_SELECT_DATA(ev, i);
 		}
 	    }
 	}
@@ -142,7 +151,7 @@ static void CallExtensionSelector(
     for (p = widget->core.event_table; p != NULL; p = p->next)
 	if (p->has_type_specifier &&
 	    EXT_TYPE(p) >= rec->min && EXT_TYPE(p) <= rec->max)
-	    count += p->mask;
+	    count = (Cardinal) (count + p->mask);
 
     if (count == 0 && !forceCall) return;
 
@@ -158,7 +167,7 @@ static void CallExtensionSelector(
 		data[count++] = EXT_SELECT_DATA(p, i);
 	    }
 
-    (*rec->proc)(widget, types, data, count, rec->client_data);
+    (*rec->proc)(widget, types, data, (int) count, rec->client_data);
     DEALLOCATE_LOCAL((char*) types);
     DEALLOCATE_LOCAL((char*) data);
 }
@@ -175,7 +184,7 @@ RemoveEventHandler(
     Boolean raw)
 {
     XtEventRec *p, **pp;
-    EventMask eventMask, oldMask = XtBuildEventMask(widget);
+    EventMask oldMask = XtBuildEventMask(widget);
 
     if (raw) raw = 1;
     pp = &widget->core.event_table;
@@ -188,7 +197,7 @@ RemoveEventHandler(
 
     /* un-register it */
     if (!has_type_specifier) {
-	eventMask = *(EventMask*)select_data;
+        EventMask eventMask = *(EventMask*)select_data;
 	eventMask &= ~NonMaskableMask;
 	if (other)
 	    eventMask |= NonMaskableMask;
@@ -222,7 +231,7 @@ RemoveEventHandler(
 	Display* dpy = XtDisplay (widget);
 
 	if (oldMask != mask)
-	    XSelectInput(dpy, XtWindow(widget), mask);
+	    XSelectInput(dpy, XtWindow(widget), (long) mask);
 
 	if (has_type_specifier) {
 	    XtPerDisplay pd = _XtGetPerDisplay(dpy);
@@ -307,7 +316,6 @@ AddEventHandler(
 	if (position == XtListHead) {
 	    p->next = widget->core.event_table;
 	    widget->core.event_table = p;
-	    pp = &widget->core.event_table;
 	} else {
 	    *pp = p;
 	    p->next = NULL;
@@ -340,9 +348,9 @@ AddEventHandler(
 		i++;
 	    if (i == p->mask) {
 		p = (XtEventRec*) XtRealloc((char*)p,
-					    sizeof(XtEventRec) +
+					    (Cardinal)(sizeof(XtEventRec) +
 					    sizeof(XtEventRecExt) +
-					    p->mask * sizeof(XtPointer));
+					    p->mask * sizeof(XtPointer)));
 		EXT_SELECT_DATA(p,i) = select_data;
 		p->mask++;
 		*pp = p;
@@ -355,7 +363,7 @@ AddEventHandler(
 	Display* dpy = XtDisplay (widget);
 
 	if (oldMask != mask)
-	    XSelectInput(dpy, XtWindow(widget), mask);
+	    XSelectInput(dpy, XtWindow(widget), (long) mask);
 
 	if (has_type_specifier) {
 	    XtPerDisplay pd = _XtGetPerDisplay (dpy);
@@ -382,7 +390,7 @@ void XtRemoveEventHandler(
     WIDGET_TO_APPCON(widget);
     LOCK_APP(app);
     RemoveEventHandler(widget, (XtPointer) &eventMask, 0, FALSE,
-		       other, proc, closure, FALSE);
+		       (Boolean) other, proc, closure, FALSE);
     UNLOCK_APP(app);
 }
 
@@ -395,7 +403,7 @@ void XtAddEventHandler(
 {
     WIDGET_TO_APPCON(widget);
     LOCK_APP(app);
-    AddEventHandler(widget, (XtPointer) &eventMask, 0, FALSE, other,
+    AddEventHandler(widget, (XtPointer) &eventMask, 0, FALSE, (Boolean) other,
 		    proc, closure, XtListTail, FALSE, FALSE);
     UNLOCK_APP(app);
 }
@@ -410,7 +418,7 @@ void XtInsertEventHandler(
 {
     WIDGET_TO_APPCON(widget);
     LOCK_APP(app);
-    AddEventHandler(widget, (XtPointer) &eventMask, 0, FALSE, other,
+    AddEventHandler(widget, (XtPointer) &eventMask, 0, FALSE, (Boolean) other,
 		    proc, closure, position, TRUE, FALSE);
     UNLOCK_APP(app);
 }
@@ -425,7 +433,7 @@ void XtRemoveRawEventHandler(
     WIDGET_TO_APPCON(widget);
     LOCK_APP(app);
     RemoveEventHandler(widget, (XtPointer) &eventMask, 0, FALSE,
-		       other, proc, closure, TRUE);
+		       (Boolean) other, proc, closure, TRUE);
     UNLOCK_APP(app);
 }
 
@@ -439,7 +447,7 @@ void XtInsertRawEventHandler(
 {
     WIDGET_TO_APPCON(widget);
     LOCK_APP(app);
-    AddEventHandler(widget, (XtPointer) &eventMask, 0, FALSE, other,
+    AddEventHandler(widget, (XtPointer) &eventMask, 0, FALSE, (Boolean) other,
 		    proc, closure, position, TRUE, TRUE);
     UNLOCK_APP(app);
 }
@@ -453,7 +461,7 @@ void XtAddRawEventHandler(
 {
     WIDGET_TO_APPCON(widget);
     LOCK_APP(app);
-    AddEventHandler(widget, (XtPointer) &eventMask, 0, FALSE, other,
+    AddEventHandler(widget, (XtPointer) &eventMask, 0, FALSE, (Boolean) other,
 		    proc, closure, XtListTail, FALSE, TRUE);
     UNLOCK_APP(app);
 }
@@ -506,7 +514,7 @@ static const WidgetRec WWfake;	/* placeholder for deletions */
 
 #define WWHASH(tab,win) ((win) & tab->mask)
 #define WWREHASHVAL(tab,win) ((((win) % tab->rehash) + 2) | 1)
-#define WWREHASH(tab,idx,rehash) ((idx + rehash) & tab->mask)
+#define WWREHASH(tab,idx,rehash) ((unsigned)(idx + rehash) & (tab->mask))
 #define WWTABLE(display) (_XtGetPerDisplay(display)->WWtable)
 
 static void ExpandWWTable(WWTable);
@@ -517,7 +525,7 @@ void XtRegisterDrawable(
     Widget widget)
 {
     WWTable tab;
-    int idx, rehash;
+    int idx;
     Widget entry;
     Window window = (Window) drawable;
     WIDGET_TO_APPCON(widget);
@@ -539,11 +547,11 @@ void XtRegisterDrawable(
     if ((tab->occupied + (tab->occupied >> 2)) > tab->mask)
 	ExpandWWTable(tab);
 
-    idx = WWHASH(tab, window);
+    idx = (int) WWHASH(tab, window);
     if ((entry = tab->entries[idx]) && entry != &WWfake) {
-	rehash = WWREHASHVAL(tab, window);
+	int rehash = (int) WWREHASHVAL(tab, window);
 	do {
-	    idx = WWREHASH(tab, idx, rehash);
+	    idx = (int) WWREHASH(tab, idx, rehash);
 	} while ((entry = tab->entries[idx]) && entry != &WWfake);
     }
     if (!entry)
@@ -560,7 +568,7 @@ void XtUnregisterDrawable(
     Drawable drawable)
 {
     WWTable tab;
-    int idx, rehash;
+    int idx;
     Widget entry;
     Window window = (Window) drawable;
     Widget widget = XtWindowToWidget (display, window);
@@ -585,12 +593,12 @@ void XtUnregisterDrawable(
 	UNLOCK_APP(app);
 	return;
     }
-    idx = WWHASH(tab, window);
+    idx = (int) WWHASH(tab, window);
     if ((entry = tab->entries[idx])) {
 	if (entry != widget) {
-	    rehash = WWREHASHVAL(tab, window);
+	    int rehash = (int) WWREHASHVAL(tab, window);
 	    do {
-		idx = WWREHASH(tab, idx, rehash);
+		idx = (int) WWREHASH(tab, idx, rehash);
 		if (!(entry = tab->entries[idx])) {
 		    UNLOCK_PROCESS;
 		    UNLOCK_APP(app);
@@ -625,11 +633,11 @@ static void ExpandWWTable(
     entries = tab->entries = (Widget *) __XtCalloc(tab->mask+1, sizeof(Widget));
     for (oldidx = 0; oldidx <= oldmask; oldidx++) {
 	if ((entry = oldentries[oldidx]) && entry != &WWfake) {
-	    newidx = WWHASH(tab, XtWindow(entry));
+	    newidx = (Cardinal) WWHASH(tab, XtWindow(entry));
 	    if (entries[newidx]) {
-		rehash = WWREHASHVAL(tab, XtWindow(entry));
+		rehash = (Cardinal) WWREHASHVAL(tab, XtWindow(entry));
 		do {
-		    newidx = WWREHASH(tab, newidx, rehash);
+		    newidx = (Cardinal) WWREHASH(tab, newidx, rehash);
 		} while (entries[newidx]);
 	    }
 	    entries[newidx] = entry;
@@ -643,9 +651,9 @@ Widget XtWindowToWidget(
     register Display *display,
     register Window window)
 {
-    register WWTable tab;
-    register int idx, rehash;
-    register Widget entry;
+    WWTable tab;
+    int idx;
+    Widget entry;
     WWPair pair;
     DPY_TO_APPCON(display);
 
@@ -654,11 +662,11 @@ Widget XtWindowToWidget(
     LOCK_APP(app);
     LOCK_PROCESS;
     tab = WWTABLE(display);
-    idx = WWHASH(tab, window);
+    idx = (int) WWHASH(tab, window);
     if ((entry = tab->entries[idx]) && XtWindow(entry) != window) {
-	rehash = WWREHASHVAL(tab, window);
+	int rehash = (int) WWREHASHVAL(tab, window);
 	do {
-	    idx = WWREHASH(tab, idx, rehash);
+	    idx = (int) WWREHASH(tab, idx, rehash);
 	} while ((entry = tab->entries[idx]) && XtWindow(entry) != window);
     }
     if (entry) {
@@ -717,8 +725,6 @@ static Boolean CallEventHandlers(
     register XtEventRec *p;
     XtEventHandler *proc;
     XtPointer *closure;
-    XtEventHandler procs[EHMAXSIZE];
-    XtPointer closures[EHMAXSIZE];
     Boolean cont_to_disp = True;
     int i, numprocs;
 
@@ -731,14 +737,10 @@ static Boolean CallEventHandlers(
 	    (p->has_type_specifier && event->type == EXT_TYPE(p)))
 	    numprocs++;
     }
-    if (numprocs > EHMAXSIZE) {
-	proc = (XtEventHandler *)__XtMalloc(numprocs * (sizeof(XtEventHandler) +
-						      sizeof(XtPointer)));
-	closure = (XtPointer *)(proc + numprocs);
-    } else {
-	proc = procs;
-	closure = closures;
-    }
+    proc = (XtEventHandler *)__XtMalloc((Cardinal)((size_t)numprocs * (sizeof(XtEventHandler) +
+						   sizeof(XtPointer))));
+    closure = (XtPointer *)(proc + numprocs);
+
     numprocs = 0;
     for (p=widget->core.event_table; p; p = p->next) {
 	if ((!p->has_type_specifier && (mask & p->mask)) ||
@@ -763,8 +765,7 @@ static Boolean CallEventHandlers(
 		*/
     for (i = 0; i < numprocs && cont_to_disp; i++)
 	(*(proc[i]))(widget, closure[i], event, &cont_to_disp);
-    if (numprocs > EHMAXSIZE)
-	XtFree((char *)proc);
+    XtFree((char *)proc);
     return cont_to_disp;
 }
 
@@ -1029,10 +1030,10 @@ void XtAddExposureToRegion(
     /* These Expose and GraphicsExpose fields are at identical offsets */
 
     if (event->type == Expose || event->type == GraphicsExpose) {
-	rect.x = ev->x;
-	rect.y = ev->y;
-	rect.width = ev->width;
-	rect.height = ev->height;
+	rect.x = (Position) ev->x;
+	rect.y = (Position) ev->y;
+	rect.width = (Dimension) ev->width;
+	rect.height = (Dimension) ev->height;
 	XUnionRectWithRegion(&rect, region, region);
     }
 }
@@ -1053,10 +1054,10 @@ static void AddExposureToRectangularRegion(
     XExposeEvent *ev = (XExposeEvent *) event;
     /* These Expose and GraphicsExpose fields are at identical offsets */
 
-    rect.x = ev->x;
-    rect.y = ev->y;
-    rect.width = ev->width;
-    rect.height = ev->height;
+    rect.x = (Position) ev->x;
+    rect.y = (Position) ev->y;
+    rect.width = (Dimension) ev->width;
+    rect.height = (Dimension) ev->height;
 
     if (XEmptyRegion(region)) {
 	XUnionRectWithRegion(&rect, region, region);
@@ -1066,10 +1067,10 @@ static void AddExposureToRectangularRegion(
 	XClipBox(region, &bbox);
 	merged.x = MIN(rect.x, bbox.x);
 	merged.y = MIN(rect.y, bbox.y);
-	merged.width = MAX(rect.x + rect.width,
-			   bbox.x + bbox.width) - merged.x;
-	merged.height = MAX(rect.y + rect.height,
-			    bbox.y + bbox.height) - merged.y;
+	merged.width = (unsigned short)(MAX(rect.x + rect.width,
+					    bbox.x + bbox.width) - merged.x);
+	merged.height = (unsigned short)(MAX(rect.y + rect.height,
+					     bbox.y + bbox.height) - merged.y);
 	XUnionRectWithRegion(&merged, region, region);
     }
 }
@@ -1323,7 +1324,7 @@ Boolean _XtDefaultDispatcher(
 	    was_dispatched = (XFilterEvent(event, XtWindow(widget))
 			      || XtDispatchEventToWidget(widget, event));
 	}
-	else was_dispatched = XFilterEvent(event, None);
+	else was_dispatched = (Boolean) XFilterEvent(event, None);
     }
     else if (grabType == pass) {
 	if (event->type == LeaveNotify ||
@@ -1351,7 +1352,7 @@ Boolean _XtDefaultDispatcher(
 
 	if ((grabList == NULL ||_XtOnGrabList(dspWidget, grabList))
 	    && XtIsSensitive(dspWidget)) {
-	    if ((was_filtered = XFilterEvent(event, XtWindow(dspWidget)))) {
+	    if ((was_filtered = (Boolean) XFilterEvent(event, XtWindow(dspWidget)))) {
 		/* If this event activated a device grab, release it. */
 		_XtUngrabBadGrabs(event, widget, mask, pdi);
 		was_dispatched = True;
@@ -1456,8 +1457,8 @@ static XtGrabRec *NewGrabRec(
     gl		      = XtNew(XtGrabRec);
     gl->next	      = NULL;
     gl->widget        = widget;
-    gl->exclusive     = exclusive;
-    gl->spring_loaded = spring_loaded;
+    XtSetBit(gl->exclusive,     exclusive);
+    XtSetBit(gl->spring_loaded, spring_loaded);
 
     return gl;
 }
@@ -1479,11 +1480,11 @@ void XtAddGrab(
 	XtAppWarningMsg(app,
 		"grabError", "xtAddGrab", XtCXtToolkitError,
 		"XtAddGrab requires exclusive grab if spring_loaded is TRUE",
-		(String *) NULL, (Cardinal *) NULL);
+		NULL, NULL);
 	exclusive = TRUE;
     }
 
-    gl = NewGrabRec(widget, exclusive, spring_loaded);
+    gl = NewGrabRec(widget, (Boolean) exclusive, (Boolean) spring_loaded);
     gl->next = *grabListPtr;
     *grabListPtr = gl;
 
@@ -1514,7 +1515,7 @@ void XtRemoveGrab(
 	    XtAppWarningMsg(app,
 		       "grabError","xtRemoveGrab",XtCXtToolkitError,
 		       "XtRemoveGrab asked to remove a widget not on the list",
-		       (String *)NULL, (Cardinal *)NULL);
+		       NULL, NULL);
 	    UNLOCK_PROCESS;
     	    UNLOCK_APP(app);
 	    return;
@@ -1541,15 +1542,19 @@ void XtMainLoop(void)
 void XtAppMainLoop(
 	XtAppContext app)
 {
-    XEvent event;
-
+    XtInputMask m = XtIMAll;
+    XtInputMask t;
     LOCK_APP(app);
     do {
-    	XtAppNextEvent(app, &event);
-#ifdef XTHREADS
-	/* assert(app == XtDisplayToApplicationContext(event.xany.display)); */
-#endif
-	XtDispatchEvent(&event);
+	if (m == 0) {
+	    m = XtIMAll;
+	    /* wait for any event, blocking */
+	    XtAppProcessEvent(app, m);
+	} else if (((t = XtAppPending(app)) & m)) {
+	    /* wait for certain events, stepping through choices */
+	    XtAppProcessEvent(app, t & m);
+	}
+	m >>= 1;
     } while(app->exit_flag == FALSE);
     UNLOCK_APP(app);
 }
@@ -1662,7 +1667,6 @@ void XtRegisterExtensionSelector(
     XtExtensionSelectProc	proc,
     XtPointer 		client_data)
 {
-    ExtSelectRec *e;
     XtPerDisplay pd;
     int i;
     DPY_TO_APPCON(dpy);
@@ -1670,14 +1674,14 @@ void XtRegisterExtensionSelector(
     if (dpy == NULL) XtErrorMsg("nullDisplay",
 		"xtRegisterExtensionSelector", XtCXtToolkitError,
 		"XtRegisterExtensionSelector requires a non-NULL display",
-		(String *) NULL, (Cardinal *) NULL);
+		NULL, NULL);
 
     LOCK_APP(app);
     LOCK_PROCESS;
     pd = _XtGetPerDisplay(dpy);
 
     for (i = 0; i < pd->ext_select_count; i++) {
-	e = &pd->ext_select_list[i];
+        ExtSelectRec *e = &pd->ext_select_list[i];
 	if (e->min == min_event_type && e->max == max_event_type) {
 	    e->proc = proc;
 	    e->client_data = client_data;
@@ -1688,7 +1692,7 @@ void XtRegisterExtensionSelector(
 	    XtErrorMsg("rangeError", "xtRegisterExtensionSelector",
 		       XtCXtToolkitError,
 	"Attempt to register multiple selectors for one extension event type",
-		       (String *) NULL, (Cardinal *) NULL);
+		       NULL, NULL);
 	    UNLOCK_PROCESS;
 	    UNLOCK_APP(app);
 	    return;
@@ -1697,7 +1701,7 @@ void XtRegisterExtensionSelector(
     pd->ext_select_count++;
     pd->ext_select_list =
 	    (ExtSelectRec *) XtRealloc((char *) pd->ext_select_list,
-		       pd->ext_select_count * sizeof(ExtSelectRec));
+		       (Cardinal) ((size_t)pd->ext_select_count * sizeof(ExtSelectRec)));
     for (i = pd->ext_select_count - 1; i > 0; i--) {
 	if (pd->ext_select_list[i-1].min > min_event_type) {
 	    pd->ext_select_list[i] = pd->ext_select_list[i-1];
