@@ -1,7 +1,5 @@
-/* $Xorg: TMgrab.c,v 1.4 2001/02/09 02:03:58 xorgcvs Exp $ */
-
 /***********************************************************
-Copyright 1993 Sun Microsystems, Inc.  All rights reserved.
+Copyright (c) 1993, Oracle and/or its affiliates. All rights reserved.
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
@@ -69,7 +67,6 @@ used in advertising or otherwise to promote the sale, use or other dealings
 in this Software without prior written authorization from The Open Group.
 
 */
-/* $XFree86: xc/lib/Xt/TMgrab.c,v 1.3 2001/12/14 19:56:30 dawes Exp $ */
 
 /*LINTLIBRARY*/
 #ifdef HAVE_CONFIG_H
@@ -105,16 +102,20 @@ static void GrabAllCorrectKeys(
 					  &careOn, &careMask);
 	if (!resolved) return;
     }
-    careOn |= modMatch->modifiers;
-    careMask |= modMatch->modifierMask;
+    careOn = (careOn | (Modifiers)modMatch->modifiers);
+    careMask = (careMask | (Modifiers)modMatch->modifierMask);
 
+    keycodes = NULL;
     XtKeysymToKeycodeList(
 	    dpy,
 	    (KeySym)typeMatch->eventCode,
 	    &keycodes,
 	    &keycount
 			 );
-    if (keycount == 0) return;
+    if (keycount == 0) {
+	XtFree((char *)keycodes);
+	return;
+    }
     for (keycodeP = keycodes; keycount--; keycodeP++) {
 	if (modMatch->standard) {
 	    /* find standard modifiers that produce this keysym */
@@ -123,8 +124,10 @@ static void GrabAllCorrectKeys(
 	    Modifiers modifiers_return;
 	    XtTranslateKeycode( dpy, *keycodeP, (Modifiers)0,
 			        &modifiers_return, &keysym );
-	    if (careOn & modifiers_return)
+	    if (careOn & modifiers_return) {
+		XtFree((char *)keycodes);
 		return;
+	    }
 	    if (keysym == typeMatch->eventCode) {
 		XtGrabKey(widget, *keycodeP, careOn,
 			  grabP->owner_events,
@@ -133,13 +136,13 @@ static void GrabAllCorrectKeys(
 			);
 		/* continue; */		/* grab all modifier combinations */
 	    }
-	    least_mod = modifiers_return & (~modifiers_return + 1);
-	    for (std_mods = modifiers_return;
+	    least_mod = (int) (modifiers_return & (~modifiers_return + 1));
+	    for (std_mods = (int) modifiers_return;
 		 std_mods >= least_mod; std_mods--) {
 		Modifiers dummy;
 		 /* check all useful combinations of modifier bits */
-		if (modifiers_return & std_mods &&
-		    !(~modifiers_return & std_mods)) {
+		if ((modifiers_return & (Modifiers)std_mods) &&
+		    !(~modifiers_return & (Modifiers)std_mods)) {
 		    XtTranslateKeycode( dpy, *keycodeP,
 					(Modifiers)std_mods,
 					&dummy, &keysym );
@@ -186,7 +189,6 @@ static Boolean DoGrab(
     TMModifierMatch	modMatch;
     Modifiers		careOn = 0;
     Modifiers		careMask = 0;
-    Boolean		resolved;
 
     LOCK_PROCESS;
     typeMatch = TMGetTypeMatch(typeIndex);
@@ -203,15 +205,15 @@ static Boolean DoGrab(
       case ButtonPress:
       case ButtonRelease:
 	if (modMatch->lateModifiers) {
-	    resolved = _XtComputeLateBindings(XtDisplay(widget),
-					      modMatch->lateModifiers,
-					      &careOn, &careMask);
+	    Boolean resolved = _XtComputeLateBindings(XtDisplay(widget),
+						      modMatch->lateModifiers,
+						      &careOn, &careMask);
 	    if (!resolved) break;
 	}
-	careOn |= modMatch->modifiers;
+	careOn = (careOn | (Modifiers) modMatch->modifiers);
 	XtGrabButton(
 		     widget,
-		     (unsigned) typeMatch->eventCode,
+		     (int) typeMatch->eventCode,
 		     careOn,
 		     grabP->owner_events,
 		     grabP->event_mask,
@@ -234,7 +236,7 @@ static Boolean DoGrab(
 	XtAppWarningMsg(XtWidgetToApplicationContext(widget),
 			"invalidPopup","unsupportedOperation",XtCXtToolkitError,
 			"Pop-up menu creation is only supported on Button, Key or EnterNotify events.",
-			(String *)NULL, (Cardinal *)NULL);
+			NULL, NULL);
 	break;
     }
     UNLOCK_PROCESS;
@@ -280,7 +282,7 @@ void _XtRegisterGrabs(
 		     */
 		    doGrab.widget = widget;
 		    doGrab.grabP = grabP;
-		    doGrab.count = count;
+		    doGrab.count = (TMShortCard) count;
 		    _XtTraverseStateTree((TMStateTree)*stateTreePtr,
 					 DoGrab,
 					 (XtPointer)&doGrab);
@@ -324,7 +326,7 @@ void XtRegisterGrabAction(
 	}
 #endif /*DEBUG*/
 
-    actionP->owner_events = owner_events;
+    actionP->owner_events = (Boolean) owner_events;
     actionP->event_mask = event_mask;
     actionP->pointer_mode = pointer_mode;
     actionP->keyboard_mode = keyboard_mode;
